<?
	require_once('guildicon.class.php');

class RyzomBanner{
	//
	// paths
	var $glPath = 'guild/'; // guild icon
	var $bgPath = 'logos/'; // banner backgrounds, faction logos
    var $fontPath = 'fonts/';
	//
	var $dateFormat = '%a, %d %b %Y %T %Z'; 
	var $lang = 'en';
	var $xml = null; // SimpleXML object
	var $banner = null; // image handler
	// translation
	var $uiName = 'Name'; 
	var $uiGuild = 'Guild'; 
	var $uiShard = 'Shard';
	var $uiStatus = 'status';
	var $uiOnline = 'online';
	var $uiOffline= 'offline'; 
	var $uiPlayed = 'played'; 
	var $uiLastSeen = 'last seen';
	// text from xml
	var $txt_title = ''; 
	var $txt_status = ''; // online / offline
	var $txt_lastSeen = ''; // last seen datetime
	var $cultLogo = ''; // filename for cult logo
	var $civLogo = ''; // .. for civilization log
	var $raceBg = ''; // .. for race background
	// banner template
	var $BANNER_W = 500;
	var $BANNER_H = 70;
	var $BANNER_BG = 'banner_bg.png';
	// margins for guild logo and text
	var $MARGIN_L = 2;
	var $MARGIN_R = 2;
	var $MARGIN_T = 2;
	var $MARGIN_B = 2;
	//
	var $GUILD_LOGO_X = 1;
	var $GUILD_LOGO_Y = 1;
	var $FONT  = 'basic.ttf';
	var $FONT_SIZE = 10.0;
	var $FONT_SMALL = 7.5;
	var $LINESPACING = 2;
	var $PADDING = 3; // for last line - space between 'status' and 'online'
	var $COL_2= 40; // x for user data column like username, guild name, etc
    var $LAST_SEEN_X = 150;
	//
	var $COLOR1=array(255, 200, 100); // color for ui elements
	var $COLOR2=array(255, 255, 255); // color for user data (white)

	/*
	 * xml - simpleXml object
	 * lng - selected language
	 */
	function RyzomBanner($xml, $lng){
        $this->FONT=$this->fontPath.$this->FONT;
		
        //
		$this->xml = $xml;
		// translation here - also make sure lng is correct
		switch($lng){
			case 'fr':
				$this->lang='fr';
				break;
			case 'de':
				$this->lang='de';
				break;
			case 'en': // fall thru
			default:
				// already defined
				$this->lang='en';
		}
		//
		$this->_parse_xml();
        // do not yet create image, do it when user asks it
		$this->banner=null;
	}
    // get banner image handler
    function getImage($rebuild=false){
		if($this->banner==null || $rebuild===true){
	        $this->banner = $this->buildImage();
		}
        return $this->banner;
    }
	//
	private function _parse_xml(){
		// title
		if(trim($this->xml->title)!=''){
			// include titles array if it exists
			if(file_exists('title_array.php')){
				include_once('title_array.php');
			}
			$title=trim($this->xml->title);
			if(isset($title_array[strtolower($title)][$this->lang])){
				$t=$title_array[strtolower($title)][$this->lang];
				$title=$t[intval($this->xml->gender)];
			}
			$this->txt_title=trim($title);
		}

		// online/ offline and login date/time format
		if(intval($this->xml->latest_login) >= intval($this->xml->latest_logout)) {
			$this->txt_status=$this->uiOnline;
			$this->txt_lastSeen = gmstrftime($this->dateFormat, intval($this->xml->latest_login));
		} else {
			$status=$uiOffline;
			$this->txt_lastSeen = gmstrftime($this->dateFormat, intval($this->xml->latest_logout));
		}
		
		// time played
		$this->txt_played=$this->seconds2text(intval($this->xml->played_time));
		
		// get declared cult icon filename
		$cult=strtolower($this->xml->declared_cult);
		switch($cult){
			case 'karavan':
				$this->cultLogo='karavan_logo_small.png';
				break;
			case 'kami':
				$this->cultLogo='kami_logo_small.png';
				break;
			default:
				$this->cultLogo=false;
		}

		// get declared civilization filename - not the same as avatar's race
		$civ=strtolower($this->xml->declared_civ);
		switch($civ){
			case 'tryker': $this->civLogo='tryker_logo_small.png';break;
			case 'matis' : $this->civLogo='matis_logo_small.png';break;
			case 'fyros' : $this->civLogo='fyros_logo_small.png';break;
			case 'zorai' : $this->civLogo='zorai_logo_small.png';break;
			default: $this->civLogo=false;
		}
		if($this->civLogo===false && $this->cultLogo===false){
			// fall back to ryzom logo - top-right corner is awfully empty
			//$this->cultLogo='ryzom_logo_small.png';
		}
		
		// race specific background
		$race=strtolower($this->xml->race);
		switch($race){
			case 'tryker': $this->raceBg='banner_bg_tryker.png';break;
			case 'matis' : $this->raceBg='banner_bg_matis.png';break;
			case 'fyros' : $this->raceBg='banner_bg_fyros.png';break;
			case 'zorai' : $this->raceBg='banner_bg_zorai.png';break;
			default: $this->raceBg=$this->BANNER_BG;
		}
		if(!file_exists($this->bgPath.$this->raceBg)){
			$this->raceBg=$this->BANNER_BG;
		}
	} // parse_xml
	// draws double text on image, making it look like it has 'shadow'
	private function _image_text_shadow($im, $size, $angle, $x, $y, $color, $font, $txt){
		$shColor = imagecolorallocate($im, 0,0,0);
		imagefttext($im, $size, $angle, $x+1, $y+1, $shColor, $font, $txt);
		imagefttext($im, $size, $angle, $x, $y, $color, $font, $txt);
	}
	// converts seconds to days/hours/minutes/seconds
	private function seconds2text($seconds){
		$neg = $secods<0;
		$seconds=abs($seconds);
		$sec = $seconds % 60;
		$min = floor(($seconds/60) % 60);
		$hour = floor(($seconds / 3600) % 24);
		$days = floor(($seconds/ 86400));
		$ret=sprintf("%02d:%02d:%02d", $hour, $min, $sec);
		if($days>0){
			if($days==1){
				$ret=$days.' day, '.$ret;
			}else{
				$ret=$days.' days, '.$ret;
			}
		}
		return $ret;
	}
//****************************************************************************
	private function buildImage(){
		$banner = imagecreatetruecolor($this->BANNER_W, $this->BANNER_H);
		$alpha=imagecolorallocatealpha($banner, 0,0,0,127);
		imagefill($banner, 0, 0, $alpha);
		imagecolortransparent($banner, $alpha);
		imagesavealpha($banner, true);

		// color
		$c1=imagecolorallocate($banner, $this->COLOR1[0], $this->COLOR1[1], $this->COLOR1[2]);
		$c2=imagecolorallocate($banner, $this->COLOR2[0], $this->COLOR2[1], $this->COLOR2[2]);

		// banner background
		if(file_exists($this->bgPath.$this->raceBg)){
			$im = imagecreatefrompng($this->bgPath.$this->raceBg);
			imagecopy($banner, $im, 0, 0, 0, 0, $this->BANNER_W, $this->BANNER_H);
			imagedestroy($im);
		}

		// Guild icon
		if(isset($this->xml->guild->name)){
			$im=GuildIcon::getImage($this->glPath, "".$this->xml->guild->icon, 'b');
			$g_logo_w=0; $g_logo_h=0;
			if($im!==false){
				$g_logo_w=imagesx($im);
				$g_logo_h=imagesy($im);
				imagecopy($banner, $im, $this->MARGIN_L+$this->GUILD_LOGO_X, $this->MARGIN_T+$this->GUILD_LOGO_Y, 0, 0, $g_logo_w, $g_logo_h);
				imagedestroy($im);
			}
		}
		
		// faction / civilization icon
		$xcultLogo=$this->BANNER_W-$this->MARGIN_R;
		if($this->cultLogo!==false && file_exists($this->bgPath.$this->cultLogo)){
			$im=imagecreatefrompng($this->bgPath.$this->cultLogo);
			$_w=imagesx($im);
			$_h=imagesy($im);
			$xcultLogo-=$_w;
			$ycultLogo=$this->MARGIN_T+$this->BANNER_H/2-$_h/2;
			imagecopy($banner, $im, $xcultLogo, $ycultLogo, 0, 0, $_w, $_h);
			imagedestroy($im);
		}
		
		if($this->civLogo!==false && file_exists($this->bgPath.$this->civLogo)){
			$im=imagecreatefrompng($this->bgPath.$this->civLogo);
			$_w=imagesx($im);
			$_h=imagesy($im);
			// put civilization logo right next to cult logo
			$xcivLogo=$xcultLogo-$_w;
			$ycivLogo=$this->MARGIN_T+$this->BANNER_H/2-$_h/2;
			imagecopy($banner, $im, $xcivLogo, $ycivLogo, 0, 0, $_w, $_h);
			imagedestroy($im);
		}

		// space between guild logo and text
		$g_logo_w+=3;
		// X location for user info 
		$col1=$g_logo_w+$this->COL_2;
			
		// name and title
		$bboxName = imageftbbox($this->FONT_SIZE, 0.0, $this->FONT, $this->uiName);
		$yName = $this->MARGIN_T+abs($bboxName[5]);
		$this->_image_text_shadow($banner, $this->FONT_SIZE, 0.0, $g_logo_w, $yName, $c1, $this->FONT, $this->uiName);
		$this->_image_text_shadow($banner,$this-> FONT_SIZE, 0.0, $col1, $yName, $c2, $this->FONT, $this->xml->name);
		if($this->txt_title!=''){
			$bboxNameTxt  = imageftbbox($this->FONT_SIZE, 0.0, $this->FONT, $this->xml->name);
			$bboxTitleTxt = imageftbbox($this->FONT_SIZE, 0.0, $this->FONT, $this->txt_title);
			//put title below name
			$this->_image_text_shadow($banner, $this->FONT_SIZE, 0.0, $col1, $yName+$this->LINESPACING+abs($bboxTitleTxt[5]), $c2, $this->FONT, $this->txt_title);
			// push guild and shard name's back
			$yName+=$this->LINESPACING+abs($bboxTitleTxt[5]);
		}
		// guild
		$yGuild=$uName+$this->LINESPACING;
		if(isset($this->xml->guild->name)){
			$bboxGuild = imageftbbox($this->FONT_SIZE, 0.0, $this->FONT, $this->uiGuild);
			$yGuild += abs($bboxGuild[5]);
			$this->_image_text_shadow($banner, $this->FONT_SIZE, 0.0, $g_logo_w, $yGuild, $c1, $this->FONT, $this->uiGuild);
			$this->_image_text_shadow($banner, $this->FONT_SIZE, 0.0, $col1,     $yGuild, $c2, $this->FONT, $this->xml->guild->name);
		}
		// shard
		$bboxShard = imageftbbox($this->FONT_SIZE, 0.0, $this->FONT, $this->uiShard);
		$yShard = $this->BANNER_H-14; // FIXME: does this fall to 'the grid' ?
		$this->_image_text_shadow($banner, $this->FONT_SIZE, 0.0, $g_logo_w, $yShard, $c1, $this->FONT, $this->uiShard);
		$this->_image_text_shadow($banner, $this->FONT_SIZE, 0.0, $col1,     $yShard, $c2, $this->FONT, $this->xml->shard);
		// status
		$bboxStatus = imageftbbox($this->FONT_SMALL, 0.0, $this->FONT, $this->uiStatus);
		$xStatus = $col1-abs($bboxStatus[2])-$this->PADDING; // align 'status' to
		$xStatusTxt=$col1;
		$yStatus = $this->BANNER_H-$this->MARGIN_B;
		$this->_image_text_shadow($banner, $this->FONT_SMALL, 0.0, $xStatus, $yStatus, $c1, $this->FONT, $this->uiStatus);
		$this->_image_text_shadow($banner, $this->FONT_SMALL, 0.0, $xStatusTxt, $yStatus, $c2, $this->FONT, $this->txt_status);
		// last seen date/time
		$bboxLastSeen = imageftbbox($this->FONT_SMALL, 0.0, $this->FONT, $this->uiLastSeen);
		$bboxLastSeenTxt = imageftbbox($this->FONT_SMALL, 0.0, $this->FONT, $this->txt_lastSeen);
		$xLastSeen = $this->LAST_SEEN_X;
		$xLastSeenTxt = $xLastSeen+abs($bboxLastSeen[2])+$this->PADDING;
		$this->_image_text_shadow($banner, $this->FONT_SMALL, 0.0, $xLastSeen,    $yStatus, $c1, $this->FONT, $this->uiLastSeen);
		$this->_image_text_shadow($banner, $this->FONT_SMALL, 0.0, $xLastSeenTxt, $yStatus, $c2, $this->FONT, $this->txt_lastSeen);
		// played time
		$bboxPlayed = imageftbbox($this->FONT_SMALL, 0.0, $this->FONT, $this->uiPlayed);
		$bboxPlayedTxt = imageftbbox($this->FONT_SMALL, 0.0, $this->FONT, $this->txt_played);
		$xPlayedTxt= $this->BANNER_W-$this->MARGIN_R-abs($bboxPlayedTxt[2]);
		$xPlayed = $xPlayedTxt-abs($bboxPlayed[2])-$this->PADDING;
		$this->_image_text_shadow($banner, $this->FONT_SMALL, 0.0, $xPlayed,    $yStatus, $c1, $this->FONT, $this->uiPlayed);
		$this->_image_text_shadow($banner, $this->FONT_SMALL, 0.0, $xPlayedTxt, $yStatus, $c2, $this->FONT, $this->txt_played);

		// ryzom.com 'link' and top border and top-right black area
		$link = 'www.ryzom.com';
		$bbox=imageftbbox(7.0, 0.0, $this->FONT, $link);
		$_w=abs($bbox[4])+5;
		$_h=abs($bbox[5])+2;
		$c=imagecolorallocate($banner, 0, 0, 0);
		imagefilledrectangle($banner, $this->BANNER_W-$_w, 0, $this->BANNER_W, $_h, $c);
		$poly=array(
			$this->BANNER_W-$_w, 0,
			$this->BANNER_W-$_w, $_h,
			$this->BANNER_W-$_w-$_h*2, 0,
		);
		imagefilledpolygon($banner, $poly, 3, $c);
		imageline($banner, 0, 0, $this->BANNER_W, 0, $c);
		$c=imagecolorallocate($banner, 255,255,255);
		$yLink= abs($bbox[5]);
		$this->_image_text_shadow($banner, 7.0, 0.0, $this->BANNER_W-abs($bbox[2])-3, $yLink, $c, $this->FONT, $link);

		// return image handler
		return $banner;
	}// buildImage
	
}// class RyzomBanner

